# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

#import serial


#serialSelected = serial.Serial('COM11', 115200, timeout=100)
#serialSelected.write(bytes('a','utf-8'))
#serialSelected.close()



#===========================================================================#
#                                                                           #
#  Copyright (C) 2006 - 2018                                                #
#  IDS Imaging Development Systems GmbH                                     #
#  Dimbacher Str. 6-8                                                       #
#  D-74182 Obersulm, Germany                                                #
#                                                                           #
#  The information in this document is subject to change without notice     #
#  and should not be construed as a commitment by IDS Imaging Development   #
#  Systems GmbH. IDS Imaging Development Systems GmbH does not assume any   #
#  responsibility for any errors that may appear in this document.          #
#                                                                           #
#  This document, or source code, is provided solely as an example          #
#  of how to utilize IDS software libraries in a sample application.        #
#  IDS Imaging Development Systems GmbH does not assume any responsibility  #
#  for the use or reliability of any portion of this document or the        #
#  described software.                                                      #
#                                                                           #
#  General permission to copy or modify, but not for profit, is hereby      #
#  granted, provided that the above copyright notice is included and        #
#  reference made to the fact that reproduction privileges were granted     #
#  by IDS Imaging Development Systems GmbH.                                 #
#                                                                           #
#  IDS Imaging Development Systems GmbH cannot assume any responsibility    #
#  for the use or misuse of any portion of this software for other than     #
#  its intended diagnostic purpose in calibrating and testing IDS           #
#  manufactured cameras and software.                                       #
#                                                                           #
#===========================================================================#

# Developer Note: I tried to let it as simple as possible.
# Therefore there are no functions asking for the newest driver software or freeing memory beforehand, etc.
# The sole purpose of this program is to show one of the simplest ways to interact with an IDS camera via the uEye API.
# (XS cameras are not supported)
#---------------------------------------------------------------------------------------------------------------------------------------

#Libraries
from pyueye import ueye
import numpy as np
import cv2
import sys
import time
import serial
from serial import Serial
import serial.tools.list_ports

#---------------------------------------------------------------------------------------------------------------------------------------

#Variables
hCam = ueye.HIDS(0)             #0: first available camera;  1-254: The camera with the specified camera ID
sInfo = ueye.SENSORINFO()
cInfo = ueye.CAMINFO()
pcImageMemory = ueye.c_mem_p()
MemID = ueye.int()
rectAOI = ueye.IS_RECT()
pitch = ueye.INT()
nBitsPerPixel = ueye.INT(24)    #24: bits per pixel for color mode; take 8 bits per pixel for monochrome
channels = 3                    #3: channels for color mode(RGB); take 1 channel for monochrome
m_nColorMode = ueye.INT()		# Y8/RGB16/RGB24/REG32
bytes_per_pixel = int(nBitsPerPixel / 8)
c=0
l=0
white_G=0
white_R=0
white_B=0
cnt_l=0
cnt_c=0

#---------------------------------------------------------------------------------------------------------------------------------------
INITIALISATION 
#---------------------------------------------------------------------------------------------------------------------------------------`

print("START")
print()


# Starts the driver and establishes the connection to the camera
nRet = ueye.is_InitCamera(hCam, None)
if nRet != ueye.IS_SUCCESS:
    print("is_InitCamera ERROR")

# Reads out the data hard-coded in the non-volatile camera memory and writes it to the data structure that cInfo points to
nRet = ueye.is_GetCameraInfo(hCam, cInfo)
if nRet != ueye.IS_SUCCESS:
    print("is_GetCameraInfo ERROR")

# You can query additional information about the sensor type used in the camera
nRet = ueye.is_GetSensorInfo(hCam, sInfo)
if nRet != ueye.IS_SUCCESS:
    print("is_GetSensorInfo ERROR")

nRet = ueye.is_ResetToDefault( hCam)
if nRet != ueye.IS_SUCCESS:
    print("is_ResetToDefault ERROR")

# Set display mode to DIB
nRet = ueye.is_SetDisplayMode(hCam, ueye.IS_SET_DM_DIB)

# Set the right color mode
if int.from_bytes(sInfo.nColorMode.value, byteorder='big') == ueye.IS_COLORMODE_BAYER:
    # setup the color depth to the current windows setting
    ueye.is_GetColorDepth(hCam, nBitsPerPixel, m_nColorMode)
    bytes_per_pixel = int(nBitsPerPixel / 8)
    print("IS_COLORMODE_BAYER: ", )
    print("\tm_nColorMode: \t\t", m_nColorMode)
    print("\tnBitsPerPixel: \t\t", nBitsPerPixel)
    print("\tbytes_per_pixel: \t\t", bytes_per_pixel)
    print()

elif int.from_bytes(sInfo.nColorMode.value, byteorder='big') == ueye.IS_COLORMODE_CBYCRY:
    # for color camera models use RGB32 mode
    m_nColorMode = ueye.IS_CM_BGRA8_PACKED
    nBitsPerPixel = ueye.INT(32)
    bytes_per_pixel = int(nBitsPerPixel / 8)
    print("IS_COLORMODE_CBYCRY: ", )
    print("\tm_nColorMode: \t\t", m_nColorMode)
    print("\tnBitsPerPixel: \t\t", nBitsPerPixel)
    print("\tbytes_per_pixel: \t\t", bytes_per_pixel)
    print()

elif int.from_bytes(sInfo.nColorMode.value, byteorder='big') == ueye.IS_COLORMODE_MONOCHROME:
    # for color camera models use RGB32 mode
    m_nColorMode = ueye.IS_CM_MONO8
    nBitsPerPixel = ueye.INT(8)
    bytes_per_pixel = int(nBitsPerPixel / 8)
    print("IS_COLORMODE_MONOCHROME: ", )
    print("\tm_nColorMode: \t\t", m_nColorMode)
    print("\tnBitsPerPixel: \t\t", nBitsPerPixel)
    print("\tbytes_per_pixel: \t\t", bytes_per_pixel)
    print()

else:
    # for monochrome camera models use Y8 mode
    m_nColorMode = ueye.IS_CM_MONO8
    nBitsPerPixel = ueye.INT(8)
    bytes_per_pixel = int(nBitsPerPixel / 8)
    print("else")

# Can be used to set the size and position of an "area of interest"(AOI) within an image
nRet = ueye.is_AOI(hCam, ueye.IS_AOI_IMAGE_GET_AOI, rectAOI, ueye.sizeof(rectAOI))
if nRet != ueye.IS_SUCCESS:
    print("is_AOI ERROR")

width = rectAOI.s32Width
height = rectAOI.s32Height

# Prints out some information about the camera and the sensor
print("Camera model:\t\t", sInfo.strSensorName.decode('utf-8'))
print("Camera serial no.:\t", cInfo.SerNo.decode('utf-8'))
print("Maximum image width:\t", width)
print("Maximum image height:\t", height)
print()

#---------------------------------------------------------------------------------------------------------------------------------------

# Allocates an image memory for an image having its dimensions defined by width and height and its color depth defined by nBitsPerPixel
nRet = ueye.is_AllocImageMem(hCam, width, height, nBitsPerPixel, pcImageMemory, MemID)
if nRet != ueye.IS_SUCCESS:
    print("is_AllocImageMem ERROR")
else:
    # Makes the specified image memory the active memory
    nRet = ueye.is_SetImageMem(hCam, pcImageMemory, MemID)
    if nRet != ueye.IS_SUCCESS:
        print("is_SetImageMem ERROR")
    else:
        # Set the desired color mode
        nRet = ueye.is_SetColorMode(hCam, m_nColorMode)



# Activates the camera's live video mode (free run mode)
nRet = ueye.is_CaptureVideo(hCam, ueye.IS_DONT_WAIT)
if nRet != ueye.IS_SUCCESS:
    print("is_CaptureVideo ERROR")

# Enables the queue mode for existing image memory sequences
nRet = ueye.is_InquireImageMem(hCam, pcImageMemory, MemID, width, height, nBitsPerPixel, pitch)
if nRet != ueye.IS_SUCCESS:
    print("is_InquireImageMem ERROR")
else:
    print("Press q to leave the programm")

#---------------------------------------------------------------------------------------------------------------------------------------
FIN INITIALISATION
#---------------------------------------------------------------------------------------------------------------------------------------
DEBUT DU CODE DE DETERMINATION DE LA COULEUR DES CUBES ET COMMUNICATION AVEC LA CARTE NUCLEO

while(nRet == ueye.IS_SUCCESS): 
    
    
    
    
    array = ueye.get_data(pcImageMemory, width, height, nBitsPerPixel, pitch, copy=False) #prend photo et la stocke dans la variable array
    frame = np.reshape(array,(height.value, width.value, bytes_per_pixel)) #Redimensionnage et stockage de l'image dans un tableau numpy  
    hsv_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV) #Conversion de l'image de RGB à HSV
    
    mask_red = cv2.inRange(hsv_frame, (170,100,100),(190,255,255)) #définition du masque rouge --> image binaire
  
    mask_green = cv2.inRange(hsv_frame, (25,20,20),(100,255,255)) #définition du masque vert --> image binaire
  
    mask_yellow = cv2.inRange(hsv_frame, (25,150,150),(35,255,255)) #définition du masque jaune --> image binaire
  
   
    #Afficher l'image initiale et celles binaires obtenues par les masques pour vérifier que les plages de couleurs choisies sont bonnes  
    #cv2.imshow("SimpleLive_Python_uEye_OpenCV", frame)  
    #cv2.imshow("jaune", mask_yellow)  
    #cv2.imshow("rouge", mask_red)  
    #cv2.imshow("vert", mask_green)  
    
    #l et c représentent les tailles des images traitées 
    l=1024 
    c=1280
    stop = 1
    white_G =0 #compteur de pixel blanc pour le masque vert
    white_R =0
    white_Y = 0
    
       
    # On compte de 10 en 10 les pixels blancs de chaque image binaire associée aux couleurs verte, jaune et rouge		
    for cnt_l in range(0,l-1,10):
        for cnt_c in range (0,c-1,10):
            if mask_green[cnt_l][cnt_c]==255:
                    white_G +=1
            if mask_red[cnt_l][cnt_c]==255:
                    white_R +=1
            if mask_yellow[cnt_l][cnt_c]==255:
                    white_Y +=1

    print(white_G)  
    print(white_R) 
    print(white_Y)
    selectPort = 5 #Port utilisé par la carte Nucélo 
           
        
    if white_G>1000 and white_G>white_R and white_G>white_Y: 
    #Si il y a plus de pixels blancs dans l'image résultante du masque vert que dans celles résultantes des deux autres masques 
    #et que le nombre de pixels blancs dépasse le seuil établi, on entre dans la boucle pour envoyer l'information 'G' --> le cube est vert

        if __name__ == "__main__":
            ports = serial.tools.list_ports.comports()
                # To obtain the list of the communication ports
            for port, desc, hwid in sorted(ports):
                print("{}: {}".format(port, desc))
                # To open the serial communication at a specific baudrate
            serNuc = Serial('COM'+str(5), 115200) #Port utilisé par la carte Nucélo = port 5
            appOk = 1
            
            if appOk:
                data_to_send = 'G'
                print(data_to_send)
                if data_to_send == 'q' or data_to_send == 'Q': #si la données à envoyer est Q, le code s'arrête
                    appOk = 0
                else:
                    serNuc.write(bytes(data_to_send, 'ascii'))
                    while serNuc.inWaiting() == 0:
                        pass
                    data_rec = serNuc.read(1)  # bytes
                    print(str(data_rec)) 
            serNuc.close() #Fin de la communication 
            
            
    if white_R>1000 and white_R>white_G and white_R>white_Y:
    #Pareil si cette fois si le cube est rouge

        if __name__ == "__main__":
            ports = serial.tools.list_ports.comports()
                # To obtain the list of the communication ports
            for port, desc, hwid in sorted(ports):
                print("{}: {}".format(port, desc))
                # To select the port to use
            #selectPort = input("Select a COM port : ")    
            #print(f"Port Selected : COM{selectPort}")
                # To open the serial communication at a specific baudrate
            serNuc = Serial('COM'+str(5), 115200)  # Under Windows only
            appOk = 1

           
            if appOk:
                data_to_send = 'R'
                print(data_to_send)
                if data_to_send == 'q' or data_to_send == 'Q':
                    appOk = 0
                else:
                    serNuc.write(bytes(data_to_send, 'ascii'))
                    while serNuc.inWaiting() == 0:
                        pass
                    data_rec = serNuc.read(1)  # bytes
                    print(str(data_rec)) 
            serNuc.close()
                    

    if white_Y>1000 and white_Y>white_G and white_Y>white_R:
     #Pareil si cette fois si le cube est rouge

        if __name__ == "__main__":
            ports = serial.tools.list_ports.comports()
                # To obtain the list of the communication ports
            for port, desc, hwid in sorted(ports):
                print("{}: {}".format(port, desc))
                # To select the port to use
            #selectPort = input("Select a COM port : ")    
            #print(f"Port Selected : COM{selectPort}")
                # To open the serial communication at a specific baudrate
            serNuc = Serial('COM'+str(5), 115200)  # Under Windows only
            appOk = 1

            
            if appOK:
                data_to_send = 'B'  #on envoie B pour jaune oui oui 
                print(data_to_send)
                if data_to_send == 'q' or data_to_send == 'Q':
                    appOk = 0
                else:
                    serNuc.write(bytes(data_to_send, 'ascii'))
                    while serNuc.inWaiting() == 0:
                        pass
                    data_rec = serNuc.read(1)  # bytes
                    print(str(data_rec)) 
            serNuc.close()
    

    
    
#---------------------------------------------------------------------------------------------------------------------------------------
FIN DE LA PARTIE COULEUR ET COMMUNICATION
#---------------------------------------------------------------------------------------------------------------------------------------
POUR QUITTER LE PROGRAMME

    # Press q if you want to end the loop
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
    
#---------------------------------------------------------------------------------------------------------------------------------------
 



# Releases an image memory that was allocated using is_AllocImageMem() and removes it from the driver management
ueye.is_FreeImageMem(hCam, pcImageMemory, MemID)

# Disables the hCam camera handle and releases the data structures and memory areas taken up by the uEye camera
ueye.is_ExitCamera(hCam)

# Destroys the OpenCv windows
cv2.destroyAllWindows()

print()
print("END")



import sys
from PyQt5.QtWidgets import QApplication, QWidget

class Fenetre(QWidget):
    def __init__(self):
        QWidget.__init__(self)
        self.setWindowTitle("Ma fenetre")

    def mousePressEvent(self, event):
        print("appui souris")

app = QApplication.instance() 
if not app:
    app = QApplication(sys.argv)
    
fen = Fenetre()
fen.show()

app.exec_()



